/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2015 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef BOUNDARY_PROFILES_HH
#define BOUNDARY_PROFILES_HH

#include "boundary/boundaryProfiles.h"

namespace plb {

template<typename T, class SurfaceData>
BoundaryProfileMap<T,SurfaceData>::BoundaryProfileMap() { }

template<typename T, class SurfaceData>
BoundaryProfileMap<T,SurfaceData>::BoundaryProfileMap(BoundaryProfile3D<T,SurfaceData>* wallProfile)
{
    add(0, wallProfile);
}

template<typename T, class SurfaceData>
BoundaryProfileMap<T,SurfaceData>::BoundaryProfileMap(BoundaryProfileMap<T,SurfaceData> const& rhs)
{
    typename std::map<plint, BoundaryProfile3D<T,SurfaceData>*>::const_iterator
        it = rhs.profiles.begin();
    for (; it != rhs.profiles.end(); ++it) {
        profiles[it->first] = it->second->clone();
    }
}

template<typename T, class SurfaceData>
BoundaryProfileMap<T,SurfaceData> const&
    BoundaryProfileMap<T,SurfaceData>::operator= (
            BoundaryProfileMap<T,SurfaceData> const& rhs )
{
    BoundaryProfileMap<T,SurfaceData>(rhs).swap(*this);
    return *this;
}

template<typename T, class SurfaceData>
void BoundaryProfileMap<T,SurfaceData>::swap(BoundaryProfileMap<T,SurfaceData>& rhs) {
    profiles.swap(rhs.profiles);
}

template<typename T, class SurfaceData>
BoundaryProfileMap<T,SurfaceData>::~BoundaryProfileMap()
{
    typename std::map<plint, BoundaryProfile3D<T,SurfaceData>*>::iterator
        it = profiles.begin();
    for (; it != profiles.end(); ++it) {
        delete it->second;
    }
}

template<typename T, class SurfaceData>
BoundaryProfileMap<T,SurfaceData>*
    BoundaryProfileMap<T,SurfaceData>::clone() const
{
    return new BoundaryProfileMap<T,SurfaceData>(*this);
}

template<typename T, class SurfaceData>
void BoundaryProfileMap<T,SurfaceData>::add(plint i, BoundaryProfile3D<T,SurfaceData>* profile)
{
    typename std::map<plint, BoundaryProfile3D<T,SurfaceData>*>::iterator
        it = profiles.find(i);
    if (it!=profiles.end()) {
        delete it->second;
    }
    profiles[i] = profile;
}

template<typename T, class SurfaceData>
void BoundaryProfileMap<T,SurfaceData>::add(plint i, BoundaryProfile3D<T,SurfaceData> const& profile)
{
    typename std::map<plint, BoundaryProfile3D<T,SurfaceData>*>::iterator
        it = profiles.find(i);
    if (it!=profiles.end()) {
        delete it->second;
    }
    profiles[i] = profile.clone();
}

template<typename T, class SurfaceData>
BoundaryProfile3D<T,SurfaceData> const&
    BoundaryProfileMap<T,SurfaceData>::operator[](plint i) const
{
    typename std::map<plint, BoundaryProfile3D<T,SurfaceData>*>::const_iterator
        it = profiles.find(i);
    PLB_ASSERT( it!=profiles.end() )
        return *it->second;
}

} // namespace plb

#endif  // BOUNDARY_PROFILES_HH

