/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2015 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ISO_SURFACE_HH
#define ISO_SURFACE_HH

#include "core/array.h"
#include "core/globalDefs.h"
#include "multiBlock/multiDataProcessorWrapper3D.h"
#include "multiBlock/multiDataProcessorWrapper3D.hh"
#include "offLattice/marchingCube.h"
#include "offLattice/marchingCube.hh"
#include "boundary/triangleMeshShape.h"
#include "boundary/triangleMeshShape.hh"
#include "geometry/triangleHashing.h"
#include "geometry/triangleHashing.hh"
#include "boundary/isoSurface.h"

namespace plb {

template<typename T>
void isoSurfaceMarchingCube (
        std::vector<typename TriangleSet<T>::Triangle>& triangles,
        ConnectedTriangleMesh<T>& mesh,
        MultiNTensorField3D<int>& voxelMatrix, Box3D const& domain )
{
    TriangleMeshShape3D<T,Array<T,3> >* meshShape = new TriangleMeshShape3D<T,Array<T,3> >(mesh);

    MultiContainerBlock3D hashContainer(voxelMatrix);
    std::vector<MultiBlock3D*> container_arg;
    container_arg.push_back(&hashContainer);
    applyProcessingFunctional (
            new MeshToHash<T>(mesh),
            hashContainer.getBoundingBox(), container_arg );

    std::vector<MultiBlock3D*> triangleMeshShapeArg;
    triangleMeshShapeArg.push_back(&voxelMatrix);
    triangleMeshShapeArg.push_back(&hashContainer);
    triangleMeshShapeArg.push_back(&voxelMatrix); // dummy argument.
    isoSurfaceMarchingCube(triangles, triangleMeshShapeArg, new BoundaryShapeIsoSurface3D<T,Array<T,3> >(meshShape), domain);
}

}  // namespace plb

#endif  // ISO_SURFACE_HH
