/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2015 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OFF_LATTICE_SETUP_3D_H
#define OFF_LATTICE_SETUP_3D_H

#include "core/globalDefs.h"
#include "core/units.h"
#include "multiBlock/group3D.h"
#include "multiBlock/coupling3D.h"
#include "atomicBlock/dataProcessingFunctional3D.h"
#include "offLattice/triangleHash.h"
#include "offLattice/voxelizer.h"
#include "offLattice/offLatticeModel3D.h"

namespace plb {

// borderWidth: This is for the voxelizer: thickness of the layers which are tagged
//   "innerBorder" and "outerBorder". A thickness of 1 is OK for Guo, which needs
//   just one layer of cells.
// blockSize: Size of blocks in the sparse/parallel representation.
//     Zero means: don't use sparse representation.
//     Negative means: default to 16 for exterior flows and 0 to interior flows.
template<typename T>
void voxelizeDomain( Group3D& group, Units3D units, RawConnectedTriangleMesh<T>& mesh,
                     int flowType, plint borderWidth=1, plint blockSize=-1, bool largeEnvelope=true );

template<typename T, template<typename U> class Descriptor>
void createFluidFields(Group3D& fields, Dynamics<T,Descriptor>* dynamics);

template<typename T, class SurfaceData>
void createOfflatticeData( Group3D& fields, ConnectedTriangleMesh<T>& obstacleMesh,
                           OffLatticeModel3D<T,SurfaceData> const& offLatticeModel );

} // namespace plb

#endif  // OFF_LATTICE_SETUP_3D_H

