/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2013 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SIMULATION_SETUP_3D_H
#define SIMULATION_SETUP_3D_H

#include "core/globalDefs.h"
#include "core/geometry3D.h"
#include "core/block3D.h"
#include "core/units.h"
#include "multiBlock/multiDataField3D.h"
#include "multiBlock/coupling3D.h"
#include "boundaryCondition/boundaryCondition3D.h"
#include "offLattice/offLatticeModel3D.h"

namespace plb {

// Instantiate the boundary conditions for the outer domain.
template<typename T, template<typename U> class Descriptor>
void outerDomainBoundaries( MultiBlockLattice3D<T,Descriptor>& lattice,
                            MultiNTensorField3D<T>& rhoBar,
                            MultiNTensorField3D<T>& j,
                            OnLatticeBoundaryCondition3D<T,Descriptor>& bc,
                            T inletVelLB, bool lateralFreeSlip );

template<typename T, template<typename U> class Descriptor, class SurfaceData>
void windTunnelingSetup_along_x (
        Actions3D& lbCycle, Group3D& fields,
        OnLatticeBoundaryCondition3D<T,Descriptor>& outerBoundaryCondition,
        OffLatticeModel3D<T,SurfaceData> const& offLatticeModel ,
        bool lateralFreeSlip, T inletVelLB, T omega, T spongeZoneFraction );

}  // namespace plb

#endif  // SIMULATION_SETUP_3D_H

