/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2013 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MESH_ALGORITHM_H
#define MESH_ALGORITHM_H

/*
 * Algorithms, applied to generic meshes.
 */

#include "core/globalDefs.h"
#include "core/units.h"
#include "geometry/triangleMesh.h"

#include <vector>
#include <string>

namespace plb {

template<typename T>    
void toLatticeUnits(TriangleMesh<T>& mesh, T dx);

template<typename T>    
void toLatticeUnits(TriangleMesh<T>& mesh, Array<T,3> const& center, T dx);

template<typename T>    
void toPhysicalUnits(TriangleMesh<T>& mesh, T dx, Array<T,3> offset = Array<T,3>(T(),T(),T()));

template<typename T>    
void toPhysicalUnits(TriangleMesh<T>& mesh, Units3D const& units);

template<typename T>
void smooth(RawConnectedTriangleMesh<T>& triangleMesh, plint maxiter=1, T relax=(T)1.0, bool isMeasureWeighted=false);

}  // namespace plb

#endif  // MESH_ALGORITHM_H

