/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2014 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef STREAMLINE_PARTICLE_WRAPPER_3D_H
#define STREAMLINE_PARTICLE_WRAPPER_3D_H

#include "core/globalDefs.h"
#include "core/array.h"
#include "core/geometry3D.h"
#include "multiBlock/multiDataField3D.h"
#include "geometry/connectedTriangleMesh.h"
#include "particles/particle3D.h"

#include <vector>
#include <string>

namespace plb {

template<typename T>
RawConnectedTriangleMesh<T> generateSphereStreamlines (
        MultiTensorField3D<T,3>& velocityField,
        std::vector<Array<T,3> > const& seedingPositions,
        bool useConstantSpatialStepIntegration, T step, T sphereRadius,
        plint sphereMinNumOfTriangles, plint numSteps,
        plint numResamplingPoints = -1 );

template<typename T>
RawConnectedTriangleMesh<T> generateSphereStreamlines (
        MultiNTensorField3D<T>& velocityField,
        std::vector<Array<T,3> > const& seedingPositions,
        bool useConstantSpatialStepIntegration, T step, T sphereRadius,
        plint sphereMinNumOfTriangles, plint numSteps,
        plint numResamplingPoints = -1 );

template<typename T>
RawConnectedTriangleMesh<T> generateTubeStreamlines (
        MultiTensorField3D<T,3>& velocityField,
        std::vector<Array<T,3> > const& seedingPositions,
        bool useConstantSpatialStepIntegration, T step, T tubeNormalRadius,
        T tubeBinormalRadius, plint tubeNumPoints, plint numSteps,
        bool useFrenetSerret, plint numResamplingPoints = -1 );

template<typename T>
RawConnectedTriangleMesh<T> generateTubeStreamlines (
        MultiNTensorField3D<T>& velocityField,
        std::vector<Array<T,3> > const& seedingPositions,
        bool useConstantSpatialStepIntegration, T step, T tubeNormalRadius,
        T tubeBinormalRadius, plint tubeNumPoints, plint numSteps,
        bool useFrenetSerret, plint numResamplingPoints = -1 );

template<typename T>
RawConnectedTriangleMesh<T>* swig_generateTubeStreamlines (
        MultiNTensorField3D<T>& velocityField,
        T* seedingPositions, int size,
        bool useConstantSpatialStepIntegration, T step, T tubeNormalRadius,
        T tubeBinormalRadius, plint tubeNumPoints, plint numSteps,
        bool useFrenetSerret, plint numResamplingPoints );

template<typename T>
RawConnectedTriangleMesh<T> generateStripStreamlines (
        MultiTensorField3D<T,3>& velocityField,
        std::vector<Array<T,3> > const& seedingPositions,
        bool useConstantSpatialStepIntegration, T step,
        T stripThickness, plint numSteps, bool useFrenetSerret,
        plint numResamplingPoints = -1 );

template<typename T>
RawConnectedTriangleMesh<T> generateStripStreamlines (
        MultiNTensorField3D<T>& velocityField,
        std::vector<Array<T,3> > const& seedingPositions,
        bool useConstantSpatialStepIntegration, T step,
        T stripThickness, plint numSteps, bool useFrenetSerret,
        plint numResamplingPoints = -1 );

template<typename T, template<typename U> class Descriptor>
RawConnectedTriangleMesh<T> generateStreamlines (
        MultiBlock3D& velocityField, // could be tensor or nTensor field.
        Particle3D<T,Descriptor>* particleTemplate,
        std::vector<Array<T,3> > const& seedingPositions,
        plint numSteps, plint numResamplingPoints, bool isNtensor );

}  // namespace plb

#endif  // STREAMLINE_PARTICLE_WRAPPER_3D_H

