/* This file is part of the Palabos library.
 *
 * Copyright (C) 2011-2015 FlowKit Sarl
 * Route d'Oron 2
 * 1010 Lausanne, Switzerland
 * E-mail contact: contact@flowkit.com
 *
 * The most recent release of Palabos can be downloaded at 
 * <http://www.palabos.org/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TRIANGLE_HASHING_H
#define TRIANGLE_HASHING_H

#include "core/globalDefs.h"
#include "atomicBlock/dataProcessingFunctional3D.h"
#include "atomicBlock/dataField3D.h"
#include "atomicBlock/atomicContainerBlock3D.h"
#include "multiBlock/multiContainerBlock3D.h"
#include "multiBlock/multiDataField3D.h"
#include "particles/particleField3D.h"
#include "geometry/triangleMesh.h"

namespace plb {

template<typename T>
class TriangleHashing {
public:
    TriangleHashing(AtomicContainerBlock3D& hashContainer);
    void assignTriangles(ConnectedTriangleMesh<T>& mesh);
    void getTriangles (
            Array<T,2> const& xRange,
            Array<T,2> const& yRange,
            Array<T,2> const& zRange,
            std::vector<plint>& foundTriangles ) const;
    void getTriangles (
            Box3D const& domain,
            std::vector<plint>& foundTriangles ) const;
private:
    ScalarField3D<std::vector<plint> >& triangles;
    std::vector<Dot3D>& assignedPositions;
};

template<typename T>
class MeshToHash : public BoxProcessingFunctional3D {
public:
    MeshToHash(ConnectedTriangleMesh<T>& mesh_);
    // Field 0: Hash.
    virtual void processGenericBlocks (
                Box3D domain, std::vector<AtomicBlock3D*> fields );
    virtual MeshToHash<T>* clone() const;
    virtual void getTypeOfModification(std::vector<modif::ModifT>& modified) const;
    virtual BlockDomain::DomainT appliesTo() const;
private:
    ConnectedTriangleMesh<T>& mesh;
};

template<typename T>
void setupHashing(MultiContainerBlock3D& container, ConnectedTriangleMesh<T>& mesh);


}  // namespace plb

#endif  // TRIANGLE_HASHING_H

